// (C) Copyright Adam Wulkiewicz 2010

//#define SPACE_EXCEPTIONS_ASSERTIONS_ENABLED
//
//#define BOOST_TEST_MODULE Global test
//
//#if defined(_MSC_VER)
//#pragma warning( disable : 4512 4996 4701 4702 )
//#endif
//#include <boost/test/included/unit_test.hpp>
//#if defined(_MSC_VER)
//#pragma warning( default : 4512 4996 4701 4702 )
//#endif
//
//#include <space/test/test.hpp>
//
//BOOST_AUTO_TEST_CASE( Main )
//{
//#if defined(_MSC_VER)
//	std::cin.get();
//#endif
//}

// -------------------------------------------------------------------------------------- //
//TEST
// -------------------------------------------------------------------------------------- //

#include <space/kdtree.hpp>

#include <iostream>

template <typename V, size_t D>
class compiletime_point
{
public:
	typedef V coordinate_type;
	static const size_t coordinate_count = D;
	template <size_t K>	inline coordinate_type const & get() const { BOOST_STATIC_ASSERT(K < coordinate_count); return c[K]; }
	template <size_t K>	inline void set(coordinate_type const& v) { BOOST_STATIC_ASSERT(K < coordinate_count); c[K] = v; }
private:
	coordinate_type c[coordinate_count];
};

// -------------------------------------------------------------------------------------- //

template <typename V, size_t D>
class runtime_point
{
public:
	typedef V coordinate_type;
	static const size_t coordinate_count = D;
	inline coordinate_type const& operator[](size_t i) const { assert(i < coordinate_count); return c[i]; }
	inline coordinate_type & operator[](size_t i) { assert(i < coordinate_count); return c[i]; }
private:
	coordinate_type c[coordinate_count];
};

// -------------------------------------------------------------------------------------- //

#include <boost/array.hpp>

namespace space
{
	template <typename T, size_t N>
	struct point_traits<boost::array<T, N> >
	{
		typedef runtime_point_array<boost::array<T, N> > point_adaptor;
	};
}

// -------------------------------------------------------------------------------------- //

#include <vector>

namespace space
{
	// -------------------------------------------------------------------------------------- //

	template <typename T>
	struct point_traits<std::vector<T> >
	{
		typedef dynamic_point_randcont<std::vector<T> > point_adaptor;
	};

	// -------------------------------------------------------------------------------------- //
}

// -------------------------------------------------------------------------------------- //

#include <boost/timer.hpp>

// -------------------------------------------------------------------------------------- //

int main()
{
	typedef compiletime_point<float, 2> ctp;
	typedef runtime_point<float, 2> rtp;
	typedef boost::array<float, 2> ap;
	typedef std::vector<float> vp;
	
	typedef space::compiletime_point<compiletime_point<float, 2> > sctp;
	typedef space::runtime_point_indexop<runtime_point<float, 2> > srtp;

#ifdef _DEBUG
	static const size_t samples = 1000;
#else
	static const size_t samples = 50000;
#endif

	std::vector<ctp> ctv;
	ctv.reserve(samples);
	std::vector<rtp> rtv;
	rtv.reserve(samples);
	std::vector<ap> av;
	av.reserve(samples);
	std::vector<vp> vv;
	vv.reserve(samples);

	std::vector<sctp> sctv;
	sctv.reserve(samples);
	std::vector<srtp> srtv;
	srtv.reserve(samples);

	for ( size_t i = 0 ; i < samples ; ++i )
	{
		int x = rand() % 100 - 50;
		int y = rand() % 100 - 50;

		ctp p1;
		p1.set<0>(float(x));
		p1.set<1>(float(y));
		ctv.push_back(p1);

		rtp p2;
		p2[0] = float(x);
		p2[1] = float(y);
		rtv.push_back(p2);

		ap p3;
		p3[0] = float(x);
		p3[1] = float(y);
		av.push_back(p3);

		vp p4(2);
		p4[0] = float(x);
		p4[1] = float(y);
		vv.push_back(p4);

		sctp p5;
		p5.set<0>(float(x));
		p5.set<1>(float(y));
		sctv.push_back(p5);

		srtp p6;
		p6.set(0, float(x));
		p6.set(1, float(y));
		srtv.push_back(p6);
	}

	for ( int i = 0 ; i < 10 ; ++i )
	{
		boost::timer t;

		space::kdtree<rtp> rtkdt(rtv.begin(), rtv.end());

		std::cout << "run-time point: " << t.elapsed() << std::endl;
		t.restart();

		space::kdtree<ctp, space::compiletime_point<ctp> > ctkdt(ctv.begin(), ctv.end());

		std::cout << "compile-time point: " << t.elapsed() << std::endl;
		t.restart();

		space::kdtree<ap> akdt(av.begin(), av.end());

		std::cout << "boost::array: " << t.elapsed() << std::endl;
		t.restart();

		space::kdtree<vp> vkdt(vv.begin(), vv.end());

		std::cout << "std::vector: " << t.elapsed() << std::endl;
		t.restart();

		space::kdtree<sctp, space::runtime_from_compiletime_point<sctp> > sctkdt(sctv.begin(), sctv.end());

		std::cout << "run-time point from compile-time point: " << t.elapsed() << std::endl;
		t.restart();

		space::kdtree<srtp, space::compiletime_from_runtime_point<srtp> > srtkdt(srtv.begin(), srtv.end());

		std::cout << "compile-time point from run-time point: " << t.elapsed() << std::endl;
	}
#if defined(_MSC_VER)
	std::cin.get();
#endif

	return 0;
}

