// (C) Copyright Adam Wulkiewicz 2010

#ifndef SPACE_ASSERT_HPP
#define SPACE_ASSERT_HPP

// -------------------------------------------------------------------------------------- //

#include <boost/static_assert.hpp>

// -------------------------------------------------------------------------------------- //

#ifdef SPACE_EXCEPTIONS_ASSERTIONS_ENABLED

#include <stdexcept>
#include <boost/lexical_cast.hpp>

// -------------------------------------------------------------------------------------- //

class assertion_error : public std::runtime_error
{
public:
	assertion_error(const std::string &message)
		: std::runtime_error(message)
	{
	}
};

inline bool assert_critical(const assertion_error &) { return true; }
inline bool assert_not_critical(const assertion_error &) { return false; }

#define SPACE_ASSERT(EXPRESSION, MESSAGE) \
	if ( !(EXPRESSION) ) \
    throw assertion_error( std::string("Assertion ( ") + #EXPRESSION + " ) failed! " + MESSAGE + ". In function: " + __FUNCTION__ + ", file: " + __FILE__ + ", line: " + boost::lexical_cast<std::string>(__LINE__) + "." )

#else

#include <cassert>

#define SPACE_ASSERT(EXPRESSION, MESSAGE) \
    assert((EXPRESSION) && (MESSAGE))

#endif

// -------------------------------------------------------------------------------------- //

#endif // SPACE_ASSERT_HPP
